function strtable = Mat2Tex(data,format,row_labels, col_labels, ErrsOrPvals, Options)
% strtable = Mat2Tex(data,format,row_labels, col_labels, ErrsOrPvals, Options)
% 
% tranforms the matrix(data) into LaTeX 'tabular' environment compatible  text
% INPUT   -- data - Matlab matrix with values,
%  			-- format: C-format string.
% 			-- row_labels, col_labels --- self_explanatory (optional - default value is {})
%         -- stderrs - either stderrs or p-values (optional)
%         -- Options Structure (optional)
%  Options
%   
% the result is the string array containing the table in LaTeX notation
% no additonal transformation for Latex is needed 
%
% EXAMPLES OF USE:
% 1.  rows = {'r1','r2','r3'}; cols = {'c1','c2'}; data=[1 2; 3 4; 5 6]; Opts.Style = 'jf'; Mat2Tex(data,'%5.3f',rows,cols, [], Opts)
%
% 2.  use of stderrs, 
%     rows = {'r1','r2','r3'}; cols = {'c1','c2'}; data=[1 2; 3 4; 5 6]; stderrs = [2 3; 4 5; 6 7]; Opts.Style = 'jf'; Mat2Tex(data,'%5.3f',rows,cols, stderrs, Opts)
%
% 3.  to skip certain positions in the final table use nan. see below: elememt (3,1) is skipped
%     rows = {'r1','r2','r3'}; cols = {'c1','c2'}; data=[1 2; 3 4; nan 6]; stderrs = [2 3; 4 5; nan 7]; Opts.Style = 'jf'; Mat2Tex(data,'%5.3f',rows,cols, stderrs, Opts)
   
  

%   Alexei Jiltsov <ajiltsov@lbs.ac.uk>, 20 September 1998
%   Requires MATLAB version 5


% modification of June, 2, 1999 allows stderrs (or t-stats) to 
% be printed just below the regression results
% it is also modified to skip NaN symbols which
% makes it convinient to present regression results 
% on different parameter sets

% modificaiton on 6  November 1999   -- implement JF style table by introducing Option
% modification on 18 September 1999  -- empty row_label are  possible  


[rc, cc] = size(data);

% ==== 

do_errs = 0;

if (nargin < 6),  Options     = []; end;
if (nargin < 5),  ErrsOrPvals = []; end;
if (nargin < 4),  col_labels  = []; end;
if (nargin < 3),  row_labels  = []; end;

eval(mat2texfillDefaultOptions(Options));

if (~isempty(ErrsOrPvals)),
   if (any(size(ErrsOrPvals) ~= [rc, cc])),
      error('mat2tex::wrong size of Errs or Pvals');
   end;
else
   do_errs = 0; do_pvals = 0;
end;   


if (iscell(format)), format = reshape(format,length(format),1); end;

if (isempty(format)),
   format = cell(cc,1);
   [format{:}] = deal('%5.2f');
elseif (size(format,1) == 1),
   tmp = format;
   format = cell(cc,1);
   [format{:}] = deal(tmp);
elseif (size(format,1)~=cc),
   error('Mat2Tex: format is not of appropriate size');
end;

rowempt = 0;
row_labels = reshape(row_labels,length(row_labels),1);
col_labels = reshape(col_labels,length(col_labels),1);

if (isempty(row_labels))
   s = '{';  rowempt =1;
elseif (size(row_labels,1)~=rc),
   error('Mat2Tex: number of row labels is not apppropriate'); 
else
   s = sprintf('{%sl',ColsSep);
end;   


% === create the alignment mode for the column (all centered as a default)

for i=1:cc,
   s = [s,sprintf('%s%s',ColsSep, ColsAlign)];
end;
s = [s,sprintf('%s}',ColsSep)];

strtable = ['\begin{tabular}',s];
strtable = strvcat(strtable,'\hline');

if (~isempty(col_labels))
   if (size(col_labels,1)~=cc),
      error('Mat2Tex: number of column labels is not apppropriate'); 
   else
      s = '';
      for i=1:cc,
         s = [s,' & ',col_labels{i}];
      end;
      s = [s,' \\'];
      s = strvcat(s,'\hline');
      strtable = strvcat(strtable,s);
   end;      
end;   

% ==== MAIN LOOP === filling the matrix by moving down row-by-row

for i =1:rc,                        % ----- row loop
   s = ''; see = '';
   naninerr = 0; 
   for j=1:cc,
      ss = sprintf(format{j}, data(i,j));
      if (isnan(data(i,j))), ss = ''; end;
      s	 = [s, ss, ' & '];
      if (do_errs),
         se  = sprintf(['(',format{j},')'], ErrsOrPvals(i,j));
         if (isnan(ErrsOrPvals(i,j))), se = '';  naninerr = naninerr + 1; end;
         see = [see, se,' & '];
      end;
      if (do_pvals),
       if (ErrsOrPvals(i,j) > PvalsCrit)
          se  = sprintf([PvalsFormat], 100*ErrsOrPvals(i,j));
       else
          se  = sprintf(['<','%d\\%%'], 100*PvalsCrit);
       end;
        if (isnan(ErrsOrPvals(i,j))), se = '';  naninerr = naninerr + 1; end;
         see = [see, se,' & '];
      end;
    end;
   sz   = size(s,2);   s    = s(1:sz-3);    s    = [s,' \\'];
   
   if (do_errs | do_pvals),
      sz   = size(see,2);   see  = see(1:sz-3);    see  = [see,' \\']; 
   end; 
   
   if (~rowempt),
      s		= [row_labels{i},' & ',s];
      if (do_errs | do_pvals ), see	= [' & ',see]; end;
   end;
   if ((do_errs | do_pvals) & naninerr < cc), % if == cc then the row consisit of nans and should be skiped
      s = strvcat(s, see, RowsSep);
   else
      s = strvcat(s,RowsSep);
   end;
   strtable = strvcat(strtable,s);
end;
if ~strcmp(RowsSep,'\hline'),
   strtable = strvcat(strtable,'\hline');
end;

strtable = strvcat(strtable,'\end{tabular}');      



% ================== OPTIONS FUNCTION =====================================================================


function [defExecLine] = mat2texfillDefaultOptions(OptIn)
%
% Purpose: returns default options for the main porgram
% if the user specifies the option by assigning the corresponding
% field then the function does not change it, but if the field of 
% the options structure is missing or 
%
%   Options:
%       --- Style     -- predefined style -- ALL other values are ignored and filled 
%                     -- accordingly to the style chosen
%       --- RowSep    -- how to separate rows, the default is '\hline' -- can be empty or something like \smallskip
%       --- ColsSep   -- how to separate cols, the default is '|' -- can be empty 
%       --- ColsAlign -- 'l','c', 'r'  -- the default is 'c'
%       --- pvalsCrit --  
% 

if isfield(OptIn, 'Style'),
   switch OptIn.Style
   case 'jf',  % jf stands for journal of finance 
      defExecLine = 'RowsSep = ''''; ColsSep = ''''; ColsAlign = ''c''; '; 
      if (isfield(OptIn,'Pvals')),
         defExecLine = [defExecLine, 'do_pvals = 1; do_errs = 0; PvalsCrit = 0.01; PvalsFormat = ''%5.1f\\%%'';    '; ];
      else
         defExecLine = [defExecLine, 'do_errs = 1; do_pvals = 0; '; ];
      end;
   otherwise,
      error('unknown table style');
   end;
end;

if isempty(OptIn),  % this is default ....
   defExecLine = 'RowSep = ''\hline''; ColsSep = ''|''; ColsAlign = ''c''; '; 
   defExecLine = [defExecLine, 'do_errs = 1; '; ];
end;

   




